#include "FormItPopupToolbar.h"
#include "TranslationDebug.h"
#include "QFormitAction.h"

#include <QAction>
#include <QToolBar>
#include <QToolButton>
#include <QLabel>
#include <QHBoxLayout>
#include <QFocusEvent>
#include <QStyle>
#include <QPainter>
#include <QSizePolicy>

#include <iostream>

FormItPopupToolbar*  FormItPopupToolbar::m_openToolbar = nullptr;
QToolButton* FormItPopupToolbar::m_trigger = nullptr;

FormItPopupToolbar::FormItPopupToolbar(QWidget* parent, const QString& labelText, const std::vector<QFormItAction*>& actions): 
    QWidget(parent),
    m_labelText(labelText)
{
    setObjectName("FormItPopupToolbar");
    setAutoFillBackground(true);
    setMouseTracking(true);
    setWindowFlags(Qt::FramelessWindowHint | Qt::Popup | Qt::NoDropShadowWindowHint);

    QHBoxLayout* layout = new QHBoxLayout();
    layout->setMargin(1);
    layout->setSpacing(0);
    
    if(!labelText.isEmpty())
    {
        QLabel* label = new QLabel(this);
        label->setText(labelText);
        layout->addWidget(label);
    }

    m_toolbar = new QToolBar(this);
    m_toolbar->setMouseTracking(true);

    for(auto pAction : actions)
    {
        auto pToolButton = new QToolButton(m_toolbar);
        pToolButton->setDefaultAction(static_cast<QAction*>(pAction));
        pToolButton->setMouseTracking(true);
        pToolButton->setProperty("hoverActive", false);
        m_toolbar->addWidget(pToolButton);
        connect(static_cast<QAction*>(pAction), &QAction::triggered, this, [this]() { this->actionClicked(); });

        m_toolButtons.push_back(pToolButton);
    }

    layout->addWidget(m_toolbar, 1);
    setLayout(layout);


    if (m_openToolbar)
        m_openToolbar->close();

    m_openToolbar = this;
}


void FormItPopupToolbar::mouseMoveEvent(QMouseEvent* _event)
{
    // For some really annoying reason, hover psuedostates do not activate
    // in popup windows in QT. I couldn't find any way to make this work,
    // so instead, apply a hover property manually.
    for(auto btn : m_toolButtons)
    {
        auto localPoint = btn->mapFromGlobal(_event->globalPos());
        if(btn->rect().contains(localPoint))
        {
            if(btn->property("hoverActive") != true)
            {
                btn->setProperty("hoverActive", true);
                btn->style()->polish(btn);
                btn->repaint();
            }
            
        }
        else
        {
            if(btn->property("hoverActive") != false)
            {
                btn->setProperty("hoverActive", false);
                btn->style()->polish(btn);
                btn->repaint();
            }
        }
    }
}

void FormItPopupToolbar::showEvent(QShowEvent* evt)
{
    
    if(m_trigger)
    {
        m_trigger->setProperty("flyoutVisible", true);
        m_trigger->style()->polish(m_trigger);
        repaint();
    }

    evt->accept();
}

void FormItPopupToolbar::closeEvent(QCloseEvent* evt)
{
    if(m_trigger)
    {
        m_trigger->setProperty("flyoutVisible", false);
        m_trigger->style()->polish(m_trigger);
        m_trigger = nullptr;
    }
    
    evt->accept();
}

void FormItPopupToolbar::paintEvent(QPaintEvent* /*evt*/)
{
    QPainter painter(this);

    if(m_trigger != nullptr)
    {
        auto bottomRight = m_trigger->mapToGlobal(QPoint(m_trigger->width()-2, m_trigger->height()+1));
        auto start = this->mapFromGlobal(bottomRight);
        auto w = width()-1;
        auto h = height()-1;
        painter.setPen(QColor(0xc2, 0xc2, 0xc2));
        painter.drawLine(start, QPoint(width(), 0));
        painter.drawLine(QPoint(w, 0), QPoint(w, h));
        painter.drawLine(QPoint(0, h), QPoint(w, h));
        painter.drawLine(QPoint(0, 0), QPoint(0, h));
    }
    
}

void FormItPopupToolbar::showFrom(QToolButton* trigger)
{
    if(trigger && (trigger->property("flyoutVisible") == true || trigger == m_trigger))
    {
        m_trigger = nullptr;
        hide();
    }
    else
    {
        m_trigger = trigger;
        show();
    }

}

void FormItPopupToolbar::actionClicked()
{
    m_openToolbar = nullptr;
    close();
}
