// Original copyright for this file, which has since been modified.
/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the QtWidgets module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/
#include "TranslationDebug.h"
#include "QtBalloonTip.h"

#include <QApplication>
#include <QBitmap>
#include <QDesktopWidget>
#include <QGridLayout>
#include <QLabel>
#include <QPainter>
#include <QPushButton>
#include <QStyle>

QWidget* QtBalloonTip::create(QWidget *pParent, QMessageBox::Icon icon,
                              const QString& title, const QString& msg, 
                              int timeout)
{
    if ((timeout < 1000) || (title.isEmpty() && msg.isEmpty()))
        return nullptr;

    return new QtBalloonTip(pParent, icon, title, msg, 
                            timeout);
}

QtBalloonTip::QtBalloonTip(QWidget *pParent, QMessageBox::Icon icon,
                           const QString& title, const QString& message, 
                           int timeout) :
    QWidget(pParent)
{
    setAttribute(Qt::WA_DeleteOnClose);

    QLabel *titleLabel = new QLabel;
    titleLabel->installEventFilter(this);
    titleLabel->setText(title);

    QFont f = titleLabel->font();
    f.setBold(true);
    titleLabel->setFont(f);
    const int iconSize = 18;
    const int closeButtonSize = 15;

    QPushButton *closeButton = new QPushButton;
    closeButton->setIcon(style()->standardIcon(QStyle::SP_TitleBarCloseButton));
    closeButton->setIconSize(QSize(closeButtonSize, closeButtonSize));
    closeButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    closeButton->setFixedSize(closeButtonSize, closeButtonSize);
    QObject::connect(closeButton, &QPushButton::clicked, this, &QtBalloonTip::close);

    QLabel *msgLabel = new QLabel;
    msgLabel->installEventFilter(this);
    msgLabel->setText(message);
    msgLabel->setTextFormat(Qt::PlainText);
    msgLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);

    // smart size for the message label
    int limit = QApplication::desktop()->availableGeometry(msgLabel).size().width() / 3;

    if (msgLabel->sizeHint().width() > limit)
        msgLabel->setWordWrap(true);

    QIcon si;
    switch (icon)
    {
        case QMessageBox::Warning:
            si = style()->standardIcon(QStyle::SP_MessageBoxWarning);
            break;
        case QMessageBox::Critical:
            si = style()->standardIcon(QStyle::SP_MessageBoxCritical);
            break;
        case QMessageBox::Information:
            si = style()->standardIcon(QStyle::SP_MessageBoxInformation);
            break;
        case QMessageBox::NoIcon:
        default:
            break;
    }

    QGridLayout *layout = new QGridLayout;

    if (!si.isNull())
    {
        QLabel *iconLabel = new QLabel;
        iconLabel->setPixmap(si.pixmap(iconSize, iconSize));
        iconLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        iconLabel->setMargin(2);
        layout->addWidget(iconLabel, 0, 0);
        layout->addWidget(titleLabel, 0, 1);
    }
    else
    {
        layout->addWidget(titleLabel, 0, 0, 1, 2);
    }

    layout->addWidget(closeButton, 0, 2);
    layout->addWidget(msgLabel, 1, 0, 1, 3);
    layout->setSizeConstraint(QLayout::SetFixedSize);
    layout->setMargin(3);
    setLayout(layout);

    QPalette pal = palette();
    pal.setColor(QPalette::Window, QColor(0xff, 0xff, 0xff));
    pal.setColor(QPalette::WindowText, Qt::black);
    setPalette(pal);

    const static int border = 1;
    const static int ah = 18, ao = 18, aw = 18, rc = 0;
    setContentsMargins(border + 3,  border + 2, border + 3, border + ah + 2);
    updateGeometry();

    QSize sz = sizeHint();
    int mr = sz.width() - 1;
    int mb = sz.height() - ah - 1;

    QPainterPath path;
    path.moveTo(rc,          0);
    path.lineTo(mr - rc,     0);
    path.arcTo (mr - rc * 2, 0,           rc * 2, rc * 2,  90, -90);
    path.lineTo(mr,          mb - rc);
    path.arcTo (mr - rc * 2, mb - rc * 2, rc * 2, rc * 2,   0, -90);
    path.lineTo(rc,          mb);
    path.arcTo (0,           mb - rc * 2, rc * 2, rc * 2, -90, -90);
    path.lineTo(0,           0 + rc);
    path.arcTo (0,           0,           rc * 2, rc * 2, 180, -90);

    // Set the mask
    QBitmap bitmap = QBitmap(sizeHint());
    bitmap.fill(Qt::color0);
    QPainter painter1(&bitmap);
    painter1.setPen(QPen(Qt::color1, border));
    painter1.setBrush(QBrush(Qt::color1));
    painter1.drawPath(path);
    setMask(bitmap);

    // Draw the border
    pixmap = QPixmap(sz);
    QPainter painter2(&pixmap);
    painter2.setPen(QPen(palette().color(QPalette::Window).darker(160), border));
    painter2.setBrush(palette().color(QPalette::Window));
    painter2.drawPath(path);

    timerId = startTimer(timeout);
    show();
}

void QtBalloonTip::paintEvent(QPaintEvent *)
{
    QPainter painter(this);
    painter.drawPixmap(rect(), pixmap);
}

void QtBalloonTip::mousePressEvent(QMouseEvent *)
{
    close();
}

void QtBalloonTip::timerEvent(QTimerEvent *e)
{
    if (e->timerId() == timerId) {
        killTimer(timerId);
        if (!underMouse())
            close();
        return;
    }
    QWidget::timerEvent(e);
}
